import * as stream from 'stream';

const BUFFER_SIZE = 16 * 1024;

export default function arrayBufferToStream(arrayBuffer: ArrayBuffer, offset = 0): stream.Readable {
  if (offset < 0 || offset + 1 >= arrayBuffer.byteLength) {
    throw new RangeError('Could not convert ArrayBuffer to ReadableStream; out of bounds.');
  }

  let position = offset;
  const outStream = new stream.Readable({
    read(size) {
      const chunkSize = Math.min(size || BUFFER_SIZE, arrayBuffer.byteLength - position); //TODO: we can probably remove BUFFER_SIZE
      let needMoreData: boolean;
      do {
        //If end is reached
        if (position + 1 >= arrayBuffer.byteLength) {
          this.push(null);
          return;
        }

        //Write chunk to stream
        const chunk = Buffer.from(arrayBuffer, position, chunkSize);
        position += chunk.length;
        needMoreData = this.push(chunk);
      } while (needMoreData);
    },
  });

  return outStream;
}
